@@ -187,7 +187,7 @@ gboolean osm_tag_key_other_value_present(tag_t *haystack, tag_t *tag) {
 gboolean osm_way_ends_with_node(way_t *way, node_t *node) {
   /* and deleted way may even not contain any nodes at all */
   /* so ignore it */
-  if(way->flags & OSM_FLAG_DELETED)
+  if(OSM_FLAGS(way) & OSM_FLAG_DELETED)
     return FALSE;
 
   /* any valid way must have at least two nodes */
@@ -204,21 +204,40 @@ gboolean osm_way_ends_with_node(way_t *way, node_t *node) {
 
 /* ------------------- node handling ------------------- */
 
-void osm_node_free(icon_t **icon, node_t *node) {
+void osm_node_free(hash_table_t *hash_table, icon_t **icon, node_t *node) {
+  item_id_t id = OSM_ID(node);
+
   if(node->icon_buf) 
     icon_free(icon, node->icon_buf);
 
   /* there must not be anything left in this chain */
   g_assert(!node->map_item_chain);
 
-  osm_tags_free(node->tag);
+  osm_tags_free(OSM_TAG(node));
+
   g_free(node);
+
+  /* also remove node from hash table */
+  if(id > 0 && hash_table) {
+    // use hash table if present
+    hash_item_t **item = &(hash_table->hash[ID2HASH(id)]);
+    while(*item) {
+      if(OSM_ID((*item)->data.node) == id) {
+	hash_item_t *cur = *item;
+	*item = (*item)->next;
+	g_free(cur);
+	return;
+      }
+      
+      item = &(*item)->next;
+    }
+  }
 }
 
-static void osm_nodes_free(icon_t **icon, node_t *node) {
+static void osm_nodes_free(hash_table_t *table, icon_t **icon, node_t *node) {
   while(node) {
     node_t *next = node->next;
-    osm_node_free(icon, node);
+    osm_node_free(table, icon, node);
     node = next;
   }
 }
@@ -227,14 +246,14 @@ void osm_node_dump(node_t *node) {
   char buf[64];
   struct tm tm;
     
-  printf("Id:      "ITEM_ID_FORMAT"\n", node->id);
-  printf("User:    %s\n", node->user?node->user->name:"<unspecified>");
-  printf("Visible: %s\n", node->visible?"yes":"no");
+  printf("Id:      "ITEM_ID_FORMAT"\n", OSM_ID(node));
+  printf("User:    %s\n", OSM_USER(node)?OSM_USER(node)->name:"<unspecified>");
+  printf("Visible: %s\n", OSM_VISIBLE(node)?"yes":"no");
   
-  localtime_r(&node->time, &tm);
+  localtime_r(&OSM_TIME(node), &tm);
   strftime(buf, sizeof(buf), "%a, %d %b %Y %H:%M:%S %Z", &tm);
   printf("Time:    %s\n", buf);
-  osm_tags_dump(node->tag);
+  osm_tags_dump(OSM_TAG(node));
 }
 
 void osm_nodes_dump(node_t *node) {
@@ -259,22 +278,40 @@ void osm_node_chain_free(node_chain_t *node_chain) {
   }
 }
 
-void osm_way_free(way_t *way) {
-  //  printf("freeing way #" ITEM_ID_FORMAT "\n", way->id);
+void osm_way_free(hash_table_t *hash_table, way_t *way) {
+  item_id_t id = OSM_ID(way);
+
+  //  printf("freeing way #" ITEM_ID_FORMAT "\n", OSM_ID(way));
 
   osm_node_chain_free(way->node_chain);
-  osm_tags_free(way->tag);
+  osm_tags_free(OSM_TAG(way));
 
   /* there must not be anything left in this chain */
   g_assert(!way->map_item_chain);
 
   g_free(way);
+
+  /* also remove way from hash table */
+  if(id > 0 && hash_table) {
+    // use hash table if present
+    hash_item_t **item = &(hash_table->hash[ID2HASH(id)]);
+    while(*item) {
+      if(OSM_ID((*item)->data.way) == id) {
+	hash_item_t *cur = *item;
+	*item = (*item)->next;
+	g_free(cur);
+	return;
+      }
+      
+      item = &(*item)->next;
+    }
+  }
 }
 
-static void osm_ways_free(way_t *way) {
+static void osm_ways_free(hash_table_t *hash_table, way_t *way) {
   while(way) {
     way_t *next = way->next;
-    osm_way_free(way);
+    osm_way_free(hash_table, way);
     way = next;
   }
 }
@@ -305,19 +342,19 @@ void osm_way_dump(way_t *way) {
   char buf[64];
   struct tm tm;
 
-  printf("Id:      "ITEM_ID_FORMAT"\n", way->id);
-  printf("User:    %s\n", way->user?way->user->name:"<unspecified>");
-  printf("Visible: %s\n", way->visible?"yes":"no");
+  printf("Id:      "ITEM_ID_FORMAT"\n", OSM_ID(way));
+  printf("User:    %s\n", OSM_USER(way)?OSM_USER(way)->name:"<unspecified>");
+  printf("Visible: %s\n", OSM_VISIBLE(way)?"yes":"no");
   node_chain_t *node_chain = way->node_chain;
   while(node_chain) {
-    printf("  Node:  "ITEM_ID_FORMAT"\n", node_chain->node->id);
+    printf("  Node:  "ITEM_ID_FORMAT"\n", OSM_ID(node_chain->node));
     node_chain = node_chain->next;
   }
   
-  localtime_r(&way->time, &tm);
+  localtime_r(&OSM_TIME(way), &tm);
   strftime(buf, sizeof(buf), "%a, %d %b %Y %H:%M:%S %Z", &tm);
   printf("Time:    %s\n", buf);
-  osm_tags_dump(way->tag);
+  osm_tags_dump(OSM_TAG(way));
 }
 
 void osm_ways_dump(way_t *way) {
@@ -366,7 +403,7 @@ void osm_members_free(member_t *member) {
 }
 
 void osm_relation_free(relation_t *relation) {
-  osm_tags_free(relation->tag);
+  osm_tags_free(OSM_TAG(relation));
   osm_members_free(relation->member);
 
   g_free(relation);
@@ -386,10 +423,10 @@ void osm_relations_dump(relation_t *relation) {
     char buf[64];
     struct tm tm;
 
-    printf("Id:      "ITEM_ID_FORMAT"\n", relation->id);
+    printf("Id:      "ITEM_ID_FORMAT"\n", OSM_ID(relation));
     printf("User:    %s\n", 
-	   relation->user?relation->user->name:"<unspecified>");
-    printf("Visible: %s\n", relation->visible?"yes":"no");
+	   OSM_USER(relation)?OSM_USER(relation)->name:"<unspecified>");
+    printf("Visible: %s\n", OSM_VISIBLE(relation)?"yes":"no");
 
     member_t *member = relation->member;
     while(member) {
@@ -403,29 +440,29 @@ void osm_relations_dump(relation_t *relation) {
       case NODE:
 	if(member->object.node)
 	  printf(" Member: Node, id = " ITEM_ID_FORMAT ", role = %s\n", 
-		 member->object.node->id, member->role);
+		 OBJECT_ID(member->object), member->role);
 	break;
 
       case WAY:
 	if(member->object.way)
 	  printf(" Member: Way, id = " ITEM_ID_FORMAT ", role = %s\n", 
-		 member->object.way->id, member->role);
+		 OBJECT_ID(member->object), member->role);
 	break;
 	
       case RELATION:
 	if(member->object.relation)
 	  printf(" Member: Relation, id = " ITEM_ID_FORMAT ", role = %s\n", 
-		 member->object.relation->id, member->role);
+		 OBJECT_ID(member->object), member->role);
 	break;
       }
 
       member = member->next;
     }
     
-    localtime_r(&relation->time, &tm);
+    localtime_r(&OSM_TIME(relation), &tm);
     strftime(buf, sizeof(buf), "%a, %d %b %Y %H:%M:%S %Z", &tm);
     printf("Time:    %s\n", buf);
-    osm_tags_dump(relation->tag);
+    osm_tags_dump(OSM_TAG(relation));
     
     printf("\n");
     relation = relation->next;
@@ -529,8 +566,8 @@ void osm_free(icon_t **icon, osm_t *osm) {
 
   if(osm->bounds)   osm_bounds_free(osm->bounds);
   if(osm->user)     osm_users_free(osm->user);
-  if(osm->way)      osm_ways_free(osm->way);
-  if(osm->node)     osm_nodes_free(icon, osm->node);
+  if(osm->way)      osm_ways_free(osm->way_hash, osm->way);
+  if(osm->node)     osm_nodes_free(osm->node_hash, icon, osm->node);
   if(osm->relation) osm_relations_free(osm->relation);
   g_free(osm);
 }
@@ -675,13 +712,13 @@ static node_t *process_node(xmlTextReaderPtr reader, osm_t *osm) {
 
   char *prop;
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "id"))) {
-    node->id = strtoul(prop, NULL, 10);
+    OSM_ID(node) = strtoul(prop, NULL, 10);
     xmlFree(prop);
   }
 
   /* new in api 0.6: */
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "version"))) {
-    node->version = strtoul(prop, NULL, 10);
+    OSM_VERSION(node) = strtoul(prop, NULL, 10);
     xmlFree(prop);
   }
 
@@ -696,17 +733,17 @@ static node_t *process_node(xmlTextReaderPtr reader, osm_t *osm) {
   }
 
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "user"))) {
-    node->user = osm_user(osm, prop);
+    OSM_USER(node) = osm_user(osm, prop);
     xmlFree(prop);
   }
 
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "visible"))) {
-    node->visible = (strcasecmp(prop, "true") == 0);
+    OSM_VISIBLE(node) = (strcasecmp(prop, "true") == 0);
     xmlFree(prop);
   }
 
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "timestamp"))) {
-    node->time = convert_iso8601(prop);
+    OSM_TIME(node) = convert_iso8601(prop);
     xmlFree(prop);
   }
 
@@ -714,7 +751,7 @@ static node_t *process_node(xmlTextReaderPtr reader, osm_t *osm) {
 
   /* append node to end of hash table if present */
   if(osm->node_hash) {
-    hash_item_t **item = &osm->node_hash->hash[ID2HASH(node->id)];
+    hash_item_t **item = &osm->node_hash->hash[ID2HASH(OSM_ID(node))];
     while(*item) item = &(*item)->next;
     
     *item = g_new0(hash_item_t, 1);
@@ -729,7 +766,7 @@ static node_t *process_node(xmlTextReaderPtr reader, osm_t *osm) {
   int depth = xmlTextReaderDepth(reader);
 
   /* scan all elements on same level or its children */
-  tag_t **tag = &node->tag;
+  tag_t **tag = &OSM_TAG(node);
   int ret = xmlTextReaderRead(reader);
   while((ret == 1) && 
 	((xmlTextReaderNodeType(reader) != XML_READER_TYPE_END_ELEMENT) ||
@@ -778,34 +815,34 @@ static way_t *process_way(xmlTextReaderPtr reader, osm_t *osm) {
 
   char *prop;
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "id"))) {
-    way->id = strtoul(prop, NULL, 10);
+    OSM_ID(way) = strtoul(prop, NULL, 10);
     xmlFree(prop);
   }
 
   /* new in api 0.6: */
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "version"))) {
-    way->version = strtoul(prop, NULL, 10);
+    OSM_VERSION(way) = strtoul(prop, NULL, 10);
     xmlFree(prop);
   }
 
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "user"))) {
-    way->user = osm_user(osm, prop);
+    OSM_USER(way) = osm_user(osm, prop);
     xmlFree(prop);
   }
 
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "visible"))) {
-    way->visible = (strcasecmp(prop, "true") == 0);
+    OSM_VISIBLE(way) = (strcasecmp(prop, "true") == 0);
     xmlFree(prop);
   }
 
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "timestamp"))) {
-    way->time = convert_iso8601(prop);
+    OSM_TIME(way) = convert_iso8601(prop);
     xmlFree(prop);
   }
 
   /* append way to end of hash table if present */
   if(osm->way_hash) {
-    hash_item_t **item = &osm->way_hash->hash[ID2HASH(way->id)];
+    hash_item_t **item = &osm->way_hash->hash[ID2HASH(OSM_ID(way))];
     while(*item) item = &(*item)->next;
     
     *item = g_new0(hash_item_t, 1);
@@ -821,7 +858,7 @@ static way_t *process_way(xmlTextReaderPtr reader, osm_t *osm) {
   int depth = xmlTextReaderDepth(reader);
 
   /* scan all elements on same level or its children */
-  tag_t **tag = &way->tag;
+  tag_t **tag = &OSM_TAG(way);
   node_chain_t **node_chain = &way->node_chain;
   int ret = xmlTextReaderRead(reader);
   while((ret == 1) && 
@@ -915,28 +952,28 @@ static relation_t *process_relation(xmlTextReaderPtr reader, osm_t *osm) {
 
   char *prop;
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "id"))) {
-    relation->id = strtoul(prop, NULL, 10);
+    OSM_ID(relation) = strtoul(prop, NULL, 10);
     xmlFree(prop);
   }
 
   /* new in api 0.6: */
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "version"))) {
-    relation->version = strtoul(prop, NULL, 10);
+    OSM_VERSION(relation) = strtoul(prop, NULL, 10);
     xmlFree(prop);
   }
 
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "user"))) {
-    relation->user = osm_user(osm, prop);
+    OSM_USER(relation) = osm_user(osm, prop);
     xmlFree(prop);
   }
 
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "visible"))) {
-    relation->visible = (strcasecmp(prop, "true") == 0);
+    OSM_VISIBLE(relation) = (strcasecmp(prop, "true") == 0);
     xmlFree(prop);
   }
 
   if((prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "timestamp"))) {
-    relation->time = convert_iso8601(prop);
+    OSM_TIME(relation) = convert_iso8601(prop);
     xmlFree(prop);
   }
 
@@ -950,7 +987,7 @@ static relation_t *process_relation(xmlTextReaderPtr reader, osm_t *osm) {
   int depth = xmlTextReaderDepth(reader);
 
   /* scan all elements on same level or its children */
-  tag_t **tag = &relation->tag;
+  tag_t **tag = &OSM_TAG(relation);
   member_t **member = &relation->member;
   int ret = xmlTextReaderRead(reader);
   while((ret == 1) && 
@@ -1124,7 +1161,7 @@ char *osm_tag_get_by_key(tag_t *tag, char *key) {
 }
 
 char *osm_way_get_value(way_t *way, char *key) {
-  tag_t *tag = way->tag;
+  tag_t *tag = OSM_TAG(way);
 
   while(tag) {
     if(strcasecmp(tag->key, key) == 0)
@@ -1137,7 +1174,7 @@ char *osm_way_get_value(way_t *way, char *key) {
 }
 
 char *osm_node_get_value(node_t *node, char *key) {
-  tag_t *tag = node->tag;
+  tag_t *tag = OSM_TAG(node);
 
   while(tag) {
     if(strcasecmp(tag->key, key) == 0)
@@ -1150,7 +1187,7 @@ char *osm_node_get_value(node_t *node, char *key) {
 }
 
 gboolean osm_way_has_value(way_t *way, char *str) {
-  tag_t *tag = way->tag;
+  tag_t *tag = OSM_TAG(way);
 
   while(tag) {
     if(tag->value && strcasecmp(tag->value, str) == 0)
@@ -1162,7 +1199,7 @@ gboolean osm_way_has_value(way_t *way, char *str) {
 }
 
 gboolean osm_node_has_value(node_t *node, char *str) {
-  tag_t *tag = node->tag;
+  tag_t *tag = OSM_TAG(node);
 
   while(tag) {
     if(tag->value && strcasecmp(tag->value, str) == 0)
@@ -1174,7 +1211,7 @@ gboolean osm_node_has_value(node_t *node, char *str) {
 }
 
 gboolean osm_node_has_tag(node_t *node) {
-  tag_t *tag = node->tag;
+  tag_t *tag = OSM_TAG(node);
 
   /* created_by tags don't count as real tags */
   if(tag && osm_is_creator_tag(tag))
@@ -1195,19 +1232,33 @@ gboolean osm_node_in_way(way_t *way, node_t *node) {
   return FALSE;
 }
 
+/* return true if node is part of other way than this one */
+gboolean osm_node_in_other_way(osm_t *osm, way_t *way, node_t *node) {
+  gboolean is_other = FALSE;
+  way_chain_t *chain = osm_node_to_way(osm, node);
+
+  while(chain) {
+    way_chain_t *next = chain->next;
+
+    if(chain->way != way)
+      is_other = TRUE;
+
+    g_free(chain);
+    chain = next;
+  }
+
+  return is_other;
+}
+
 static void osm_generate_tags(tag_t *tag, xmlNodePtr node) {
   while(tag) {
+    /* skip "created_by" tags as they aren't needed anymore with api 0.6 */
     /* make sure "created_by" tag contains our id */
-    if(strcasecmp(tag->key, "created_by") == 0) {
-      if(strcasecmp(tag->value, PACKAGE " v" VERSION) != 0) {      
-	g_free(tag->value);
-	tag->value = g_strdup(PACKAGE " v" VERSION);
-      }
+    if(strcasecmp(tag->key, "created_by") != 0) {
+      xmlNodePtr tag_node = xmlNewChild(node, NULL, BAD_CAST "tag", NULL);
+      xmlNewProp(tag_node, BAD_CAST "k", BAD_CAST tag->key);
+      xmlNewProp(tag_node, BAD_CAST "v", BAD_CAST tag->value);
     }
-
-    xmlNodePtr tag_node = xmlNewChild(node, NULL, BAD_CAST "tag", NULL);
-    xmlNewProp(tag_node, BAD_CAST "k", BAD_CAST tag->key);
-    xmlNewProp(tag_node, BAD_CAST "v", BAD_CAST tag->value);
     tag = tag->next;
   }
 }
@@ -1232,11 +1283,11 @@ static char *osm_generate_xml(osm_t *osm, item_id_t changeset,
       xmlNodePtr node_node = xmlNewChild(root_node, NULL, 
 					 BAD_CAST "node", NULL);
       /* new nodes don't have an id, but get one after the upload */
-      if(!(node->flags & OSM_FLAG_NEW)) {
-	snprintf(str, sizeof(str), "%u", (unsigned)node->id);
+      if(!(OSM_FLAGS(node) & OSM_FLAG_NEW)) {
+	snprintf(str, sizeof(str), ITEM_ID_FORMAT, OSM_ID(node));
 	xmlNewProp(node_node, BAD_CAST "id", BAD_CAST str);
       }
-      snprintf(str, sizeof(str), "%u", (unsigned)node->version);
+      snprintf(str, sizeof(str), "%u", OSM_VERSION(node));
       xmlNewProp(node_node, BAD_CAST "version", BAD_CAST str);
       snprintf(str, sizeof(str), "%u", (unsigned)changeset);
       xmlNewProp(node_node, BAD_CAST "changeset", BAD_CAST str);
@@ -1244,7 +1295,7 @@ static char *osm_generate_xml(osm_t *osm, item_id_t changeset,
       xmlNewProp(node_node, BAD_CAST "lat", BAD_CAST str);
       g_ascii_formatd(str, sizeof(str), LL_FORMAT, node->pos.lon);
       xmlNewProp(node_node, BAD_CAST "lon", BAD_CAST str);    
-      osm_generate_tags(node->tag, node_node);
+      osm_generate_tags(OSM_TAG(node), node_node);
     }
     break;
 
@@ -1252,9 +1303,9 @@ static char *osm_generate_xml(osm_t *osm, item_id_t changeset,
     {
       way_t *way = (way_t*)item;
       xmlNodePtr way_node = xmlNewChild(root_node, NULL, BAD_CAST "way", NULL);
-      snprintf(str, sizeof(str), "%u", (unsigned)way->id);
+      snprintf(str, sizeof(str), ITEM_ID_FORMAT, OSM_ID(way));
       xmlNewProp(way_node, BAD_CAST "id", BAD_CAST str);
-      snprintf(str, sizeof(str), "%u", (unsigned)way->version);
+      snprintf(str, sizeof(str), "%u", OSM_VERSION(way));
       xmlNewProp(way_node, BAD_CAST "version", BAD_CAST str);
       snprintf(str, sizeof(str), "%u", (unsigned)changeset);
       xmlNewProp(way_node, BAD_CAST "changeset", BAD_CAST str);
@@ -1262,13 +1313,13 @@ static char *osm_generate_xml(osm_t *osm, item_id_t changeset,
       node_chain_t *node_chain = way->node_chain;
       while(node_chain) {
 	xmlNodePtr nd_node = xmlNewChild(way_node, NULL, BAD_CAST "nd", NULL);
-	char *str = g_strdup_printf(ITEM_ID_FORMAT, node_chain->node->id);
+	char *str = g_strdup_printf(ITEM_ID_FORMAT, OSM_ID(node_chain->node));
 	xmlNewProp(nd_node, BAD_CAST "ref", BAD_CAST str);
 	g_free(str);
 	node_chain = node_chain->next;
       }
       
-      osm_generate_tags(way->tag, way_node);
+      osm_generate_tags(OSM_TAG(way), way_node);
     } 
     break;
 
@@ -1277,9 +1328,9 @@ static char *osm_generate_xml(osm_t *osm, item_id_t changeset,
       relation_t *relation = (relation_t*)item;
       xmlNodePtr rel_node = xmlNewChild(root_node, NULL, 
 					BAD_CAST "relation", NULL);
-      snprintf(str, sizeof(str), "%u", (unsigned)relation->id);
+      snprintf(str, sizeof(str), ITEM_ID_FORMAT, OSM_ID(relation));
       xmlNewProp(rel_node, BAD_CAST "id", BAD_CAST str);
-      snprintf(str, sizeof(str), "%u", (unsigned)relation->version);
+      snprintf(str, sizeof(str), "%u", OSM_VERSION(relation));
       xmlNewProp(rel_node, BAD_CAST "version", BAD_CAST str);
       snprintf(str, sizeof(str), "%u", (unsigned)changeset);
       xmlNewProp(rel_node, BAD_CAST "changeset", BAD_CAST str);
@@ -1287,22 +1338,19 @@ static char *osm_generate_xml(osm_t *osm, item_id_t changeset,
       member_t *member = relation->member;
       while(member) {
 	xmlNodePtr m_node = xmlNewChild(rel_node,NULL,BAD_CAST "member", NULL);
-	char *str = NULL;
+	char *str = g_strdup_printf(ITEM_ID_FORMAT, OBJECT_ID(member->object));
 
 	switch(member->object.type) {
 	case NODE:
 	  xmlNewProp(m_node, BAD_CAST "type", BAD_CAST "node");
-	  str = g_strdup_printf(ITEM_ID_FORMAT, member->object.node->id);
 	  break;
 
 	case WAY:
 	  xmlNewProp(m_node, BAD_CAST "type", BAD_CAST "way");
-	  str = g_strdup_printf(ITEM_ID_FORMAT, member->object.way->id);
 	  break;
 
 	case RELATION:
 	  xmlNewProp(m_node, BAD_CAST "type", BAD_CAST "relation");
-	  str = g_strdup_printf(ITEM_ID_FORMAT, member->object.relation->id);
 	  break;
 
 	default:
@@ -1321,7 +1369,7 @@ static char *osm_generate_xml(osm_t *osm, item_id_t changeset,
 
 	member = member->next;
       }
-      osm_generate_tags(relation->tag, rel_node);
+      osm_generate_tags(OSM_TAG(relation), rel_node);
     }
     break;
 
@@ -1381,9 +1429,6 @@ char *osm_generate_xml_changeset(osm_t *osm, char *comment) {
   xmlFreeDoc(doc);
   xmlCleanupParser();
 
-  //  puts("xml encoding result:");
-  //  puts((char*)result);
-
   return (char*)result;
 }
 
@@ -1395,7 +1440,7 @@ node_t *osm_get_node_by_id(osm_t *osm, item_id_t id) {
     // use hash table if present
     hash_item_t *item = osm->node_hash->hash[ID2HASH(id)];
     while(item) {
-      if(item->data.node->id == id)
+      if(OSM_ID(item->data.node) == id)
 	return item->data.node;
       
       item = item->next;
@@ -1405,7 +1450,7 @@ node_t *osm_get_node_by_id(osm_t *osm, item_id_t id) {
   /* use linear search if no hash tables are present or search in hash table failed */
   node_t *node = osm->node;
   while(node) {
-    if(node->id == id)
+    if(OSM_ID(node) == id)
       return node;
     
     node = node->next;
@@ -1419,7 +1464,7 @@ way_t *osm_get_way_by_id(osm_t *osm, item_id_t id) {
     // use hash table if present
     hash_item_t *item = osm->way_hash->hash[ID2HASH(id)];
     while(item) {
-      if(item->data.way->id == id)
+      if(OSM_ID(item->data.way) == id)
 	return item->data.way;
       
       item = item->next;
@@ -1429,7 +1474,7 @@ way_t *osm_get_way_by_id(osm_t *osm, item_id_t id) {
   /* use linear search if no hash tables are present or search on hash table failed */
   way_t *way = osm->way;
   while(way) {
-    if(way->id == id)
+    if(OSM_ID(way) == id)
       return way;
     
     way = way->next;
@@ -1442,7 +1487,7 @@ relation_t *osm_get_relation_by_id(osm_t *osm, item_id_t id) {
   // use linear search
   relation_t *relation = osm->relation;
   while(relation) {
-    if(relation->id == id)
+    if(OSM_ID(relation) == id)
       return relation;
 
     relation = relation->next;
@@ -1460,7 +1505,7 @@ item_id_t osm_new_way_id(osm_t *osm) {
     gboolean found = FALSE;
     way_t *way = osm->way;
     while(way) {
-      if(way->id == id)
+      if(OSM_ID(way) == id)
 	found = TRUE;
 
       way = way->next;
@@ -1482,7 +1527,7 @@ item_id_t osm_new_node_id(osm_t *osm) {
     gboolean found = FALSE;
     node_t *node = osm->node;
     while(node) {
-      if(node->id == id)
+      if(OSM_ID(node) == id)
 	found = TRUE;
 
       node = node->next;
@@ -1504,7 +1549,7 @@ item_id_t osm_new_relation_id(osm_t *osm) {
     gboolean found = FALSE;
     relation_t *relation = osm->relation;
     while(relation) {
-      if(relation->id == id)
+      if(OSM_ID(relation) == id)
 	found = TRUE;
 
       relation = relation->next;
@@ -1523,11 +1568,11 @@ node_t *osm_node_new(osm_t *osm, gint x, gint y) {
   printf("Creating new node\n");
 
   node_t *node = g_new0(node_t, 1);
-  node->version = 1;
+  OSM_VERSION(node) = 1;
   node->lpos.x = x; 
   node->lpos.y = y;
-  node->visible = TRUE;
-  node->time = time(NULL);
+  OSM_VISIBLE(node) = TRUE;
+  OSM_TIME(node) = time(NULL);
 
   /* convert screen position back to ll */
   lpos2pos(osm->bounds, &node->lpos, &node->pos);
@@ -1542,8 +1587,8 @@ node_t *osm_node_new(osm_t *osm, gint x, gint y) {
 void osm_node_attach(osm_t *osm, node_t *node) {
   printf("Attaching node\n");
 
-  node->id = osm_new_node_id(osm);
-  node->flags = OSM_FLAG_NEW;
+  OSM_ID(node) = osm_new_node_id(osm);
+  OSM_FLAGS(node) = OSM_FLAG_NEW;
 
   /* attach to end of node list */
   node_t **lnode = &osm->node;
@@ -1564,10 +1609,10 @@ way_t *osm_way_new(void) {
   printf("Creating new way\n");
 
   way_t *way = g_new0(way_t, 1);
-  way->version = 1;
-  way->visible = TRUE;
-  way->flags = OSM_FLAG_NEW;
-  way->time = time(NULL);
+  OSM_VERSION(way) = 1;
+  OSM_VISIBLE(way) = TRUE;
+  OSM_FLAGS(way) = OSM_FLAG_NEW;
+  OSM_TIME(way) = time(NULL);
 
   return way;
 }
@@ -1575,8 +1620,8 @@ way_t *osm_way_new(void) {
 void osm_way_attach(osm_t *osm, way_t *way) {
   printf("Attaching way\n");
 
-  way->id = osm_new_way_id(osm);
-  way->flags = OSM_FLAG_NEW;
+  OSM_ID(way) = osm_new_way_id(osm);
+  OSM_FLAGS(way) = OSM_FLAG_NEW;
 
   /* attach to end of way list */
   way_t **lway = &osm->way;
@@ -1584,6 +1629,35 @@ void osm_way_attach(osm_t *osm, way_t *way) {
   *lway = way;
 }
 
+void osm_way_restore(osm_t *osm, way_t *way, item_id_chain_t *id_chain) {
+  printf("Restoring way\n");
+
+  /* attach to end of node list */
+  way_t **lway = &osm->way;
+  while(*lway) lway = &(*lway)->next;  
+  *lway = way;
+
+  /* restore node memberships by converting ids into real pointers */
+  g_assert(!way->node_chain);
+  node_chain_t **node_chain = &(way->node_chain);
+  while(id_chain) {
+    item_id_chain_t *id_next = id_chain->next;
+    printf("Node "ITEM_ID_FORMAT" is member\n", id_chain->id);
+
+    *node_chain = g_new0(node_chain_t, 1);
+    (*node_chain)->node = osm_get_node_by_id(osm, id_chain->id);
+    (*node_chain)->node->ways++;
+
+    printf("   -> %p\n", (*node_chain)->node);
+
+    g_free(id_chain);
+    id_chain = id_next;
+    node_chain = &(*node_chain)->next;
+  }
+
+  printf("done\n");
+}
+
 /* returns pointer to chain of ways affected by this deletion */
 way_chain_t *osm_node_delete(osm_t *osm, icon_t **icon, 
 			     node_t *node, gboolean permanently,
@@ -1591,9 +1665,9 @@ way_chain_t *osm_node_delete(osm_t *osm, icon_t **icon,
   way_chain_t *way_chain = NULL, **cur_way_chain = &way_chain;
 
   /* new nodes aren't stored on the server and are just deleted permanently */
-  if(node->flags & OSM_FLAG_NEW) {
+  if(OSM_FLAGS(node) & OSM_FLAG_NEW) {
     printf("About to delete NEW node #" ITEM_ID_FORMAT 
-	   " -> force permanent delete\n", node->id);
+	   " -> force permanent delete\n", OSM_ID(node));
     permanently = TRUE;
   }
 
@@ -1617,7 +1691,7 @@ way_chain_t *osm_node_delete(osm_t *osm, icon_t **icon,
     }
 
     if(modified) {
-      way->flags |= OSM_FLAG_DIRTY;
+      OSM_FLAGS(way) |= OSM_FLAG_DIRTY;
       
       /* and add the way to the list of affected ways */
       *cur_way_chain = g_new0(way_chain_t, 1);
@@ -1629,10 +1703,10 @@ way_chain_t *osm_node_delete(osm_t *osm, icon_t **icon,
   }
 
   if(!permanently) {
-    printf("mark node #" ITEM_ID_FORMAT " as deleted\n", node->id);
-    node->flags |= OSM_FLAG_DELETED;
+    printf("mark node #" ITEM_ID_FORMAT " as deleted\n", OSM_ID(node));
+    OSM_FLAGS(node) |= OSM_FLAG_DELETED;
   } else {
-    printf("permanently delete node #" ITEM_ID_FORMAT "\n", node->id);
+    printf("permanently delete node #" ITEM_ID_FORMAT "\n", OSM_ID(node));
 
     /* remove it from the chain */
     node_t **cnode = &osm->node;
@@ -1643,7 +1717,8 @@ way_chain_t *osm_node_delete(osm_t *osm, icon_t **icon,
 	found++;
 	*cnode = (*cnode)->next;
 
-	osm_node_free(icon, node);
+	g_assert(osm);
+	osm_node_free(osm->node_hash, icon, node);
       } else
 	cnode = &((*cnode)->next);
     }
@@ -1664,7 +1739,8 @@ guint osm_way_number_of_nodes(way_t *way) {
 }
 
 /* return all relations a node is in */
-relation_chain_t *osm_node_to_relation(osm_t *osm, node_t *node) {
+relation_chain_t *osm_node_to_relation(osm_t *osm, node_t *node, 
+				       gboolean via_way) {
   relation_chain_t *rel_chain = NULL, **cur_rel_chain = &rel_chain;
 
   relation_t *relation = osm->relation;
@@ -1680,16 +1756,18 @@ relation_chain_t *osm_node_to_relation(osm_t *osm, node_t *node) {
 	  is_member = TRUE;
 	break;
 
-      case WAY: {
-	/* ways have to be checked for the nodes they consist of */
-	node_chain_t *chain = member->object.way->node_chain;
-	while(chain && !is_member) {
-	  if(chain->node == node)
-	    is_member = TRUE;
-
-	  chain = chain->next;
+      case WAY: 
+	if(via_way) {
+	  /* ways have to be checked for the nodes they consist of */
+	  node_chain_t *chain = member->object.way->node_chain;
+	  while(chain && !is_member) {
+	    if(chain->node == node)
+	      is_member = TRUE;
+	    
+	    chain = chain->next;
+	  }
 	}
-      } break;
+	break;
 
       default:
 	break;
@@ -1788,7 +1866,7 @@ relation_chain_t *osm_object_to_relation(osm_t *osm, object_t *object) {
 
   switch(object->type) {
   case NODE:
-    rel_chain = osm_node_to_relation(osm, object->node);
+    rel_chain = osm_node_to_relation(osm, object->node, FALSE);
     break;
 
   case WAY:
@@ -1853,7 +1931,7 @@ gboolean osm_position_within_bounds(osm_t *osm, gint x, gint y) {
 /* be deleted */
 void osm_node_remove_from_relation(osm_t *osm, node_t *node) {
   relation_t *relation = osm->relation;
-  printf("removing node #" ITEM_ID_FORMAT " from all relations:\n", node->id);
+  printf("removing node #" ITEM_ID_FORMAT " from all relations:\n", OSM_ID(node));
 
   while(relation) {
     member_t **member = &relation->member;
@@ -1861,13 +1939,13 @@ void osm_node_remove_from_relation(osm_t *osm, node_t *node) {
       if(((*member)->object.type == NODE) &&
 	 ((*member)->object.node == node)) {
 
-	printf("  from relation #" ITEM_ID_FORMAT "\n", relation->id);
+	printf("  from relation #" ITEM_ID_FORMAT "\n", OSM_ID(relation));
 	
 	member_t *cur = *member;
 	*member = (*member)->next;
 	osm_member_free(cur);
 
-	relation->flags |= OSM_FLAG_DIRTY;
+	OSM_FLAGS(relation) |= OSM_FLAG_DIRTY;
       } else
 	member = &(*member)->next;
     }
@@ -1878,7 +1956,7 @@ void osm_node_remove_from_relation(osm_t *osm, node_t *node) {
 /* remove the given way from all relations */
 void osm_way_remove_from_relation(osm_t *osm, way_t *way) {
   relation_t *relation = osm->relation;
-  printf("removing way #" ITEM_ID_FORMAT " from all relations:\n", way->id);
+  printf("removing way #" ITEM_ID_FORMAT " from all relations:\n", OSM_ID(way));
 
   while(relation) {
     member_t **member = &relation->member;
@@ -1886,13 +1964,13 @@ void osm_way_remove_from_relation(osm_t *osm, way_t *way) {
       if(((*member)->object.type == WAY) &&
 	 ((*member)->object.way == way)) {
 
-	printf("  from relation #" ITEM_ID_FORMAT "\n", relation->id);
+	printf("  from relation #" ITEM_ID_FORMAT "\n", OSM_ID(relation));
 	
 	member_t *cur = *member;
 	*member = (*member)->next;
 	osm_member_free(cur);
 
-	relation->flags |= OSM_FLAG_DIRTY;
+	OSM_FLAGS(relation) |= OSM_FLAG_DIRTY;
       } else
 	member = &(*member)->next;
     }
@@ -1904,10 +1982,10 @@ relation_t *osm_relation_new(void) {
   printf("Creating new relation\n");
 
   relation_t *relation = g_new0(relation_t, 1);
-  relation->version = 1;
-  relation->visible = TRUE;
-  relation->flags = OSM_FLAG_NEW;
-  relation->time = time(NULL);
+  OSM_VERSION(relation) = 1;
+  OSM_VISIBLE(relation) = TRUE;
+  OSM_FLAGS(relation) = OSM_FLAG_NEW;
+  OSM_TIME(relation) = time(NULL);
 
   return relation;
 }
@@ -1915,8 +1993,8 @@ relation_t *osm_relation_new(void) {
 void osm_relation_attach(osm_t *osm, relation_t *relation) {
   printf("Attaching relation\n");
 
-  relation->id = osm_new_relation_id(osm);
-  relation->flags = OSM_FLAG_NEW;
+  OSM_ID(relation) = osm_new_relation_id(osm);
+  OSM_FLAGS(relation) = OSM_FLAG_NEW;
 
   /* attach to end of relation list */
   relation_t **lrelation = &osm->relation;
@@ -1929,9 +2007,9 @@ void osm_way_delete(osm_t *osm, icon_t **icon,
 		    way_t *way, gboolean permanently) {
 
   /* new ways aren't stored on the server and are just deleted permanently */
-  if(way->flags & OSM_FLAG_NEW) {
+  if(OSM_FLAGS(way) & OSM_FLAG_NEW) {
     printf("About to delete NEW way #" ITEM_ID_FORMAT 
-	   " -> force permanent delete\n", way->id);
+	   " -> force permanent delete\n", OSM_ID(way));
     permanently = TRUE;
   }
 
@@ -1941,7 +2019,7 @@ void osm_way_delete(osm_t *osm, icon_t **icon,
 
     (*chain)->node->ways--;
     printf("checking node #" ITEM_ID_FORMAT " (still used by %d)\n", 
-	   (*chain)->node->id, (*chain)->node->ways);
+	   OSM_ID((*chain)->node), (*chain)->node->ways);
 
     /* this node must only be part of this way */
     if(!(*chain)->node->ways) {
@@ -1966,10 +2044,10 @@ void osm_way_delete(osm_t *osm, icon_t **icon,
   way->node_chain = NULL;
 
   if(!permanently) {
-    printf("mark way #" ITEM_ID_FORMAT " as deleted\n", way->id);
-    way->flags |= OSM_FLAG_DELETED;
+    printf("mark way #" ITEM_ID_FORMAT " as deleted\n", OSM_ID(way));
+    OSM_FLAGS(way) |= OSM_FLAG_DELETED;
   } else {
-    printf("permanently delete way #" ITEM_ID_FORMAT "\n", way->id);
+    printf("permanently delete way #" ITEM_ID_FORMAT "\n", OSM_ID(way));
 
     /* remove it from the chain */
     way_t **cway = &osm->way;
@@ -1980,7 +2058,8 @@ void osm_way_delete(osm_t *osm, icon_t **icon,
 	found++;
 	*cway = (*cway)->next;
 
-	osm_way_free(way);
+	g_assert(osm);
+	osm_way_free(osm->way_hash, way);
       } else
 	cway = &((*cway)->next);
     }
@@ -1993,9 +2072,9 @@ void osm_relation_delete(osm_t *osm, relation_t *relation,
 
   /* new relations aren't stored on the server and are just */
   /* deleted permanently */
-  if(relation->flags & OSM_FLAG_NEW) {
+  if(OSM_FLAGS(relation) & OSM_FLAG_NEW) {
     printf("About to delete NEW relation #" ITEM_ID_FORMAT 
-	   " -> force permanent delete\n", relation->id);
+	   " -> force permanent delete\n", OSM_ID(relation));
     permanently = TRUE;
   }
 
@@ -2003,10 +2082,11 @@ void osm_relation_delete(osm_t *osm, relation_t *relation,
   /* don't have any reference to the relation they are part of */
 
   if(!permanently) {
-    printf("mark relation #" ITEM_ID_FORMAT " as deleted\n", relation->id);
-    relation->flags |= OSM_FLAG_DELETED;
+    printf("mark relation #" ITEM_ID_FORMAT " as deleted\n", OSM_ID(relation));
+    OSM_FLAGS(relation) |= OSM_FLAG_DELETED;
   } else {
-    printf("permanently delete relation #" ITEM_ID_FORMAT "\n", relation->id);
+    printf("permanently delete relation #" ITEM_ID_FORMAT "\n", 
+	   OSM_ID(relation));
 
     /* remove it from the chain */
     relation_t **crelation = &osm->relation;
@@ -2053,7 +2133,7 @@ static const char *DS_RIGHT_SUFFIX = ":right";
 
 guint
 osm_way_reverse_direction_sensitive_tags (way_t *way) {
-  tag_t *tag = way->tag;
+  tag_t *tag = OSM_TAG(way);
   guint n_tags_altered = 0;
   while (tag != NULL) {
     char *lc_key = g_ascii_strdown(tag->key, -1);
@@ -2106,7 +2186,7 @@ osm_way_reverse_direction_sensitive_tags (way_t *way) {
     tag = tag->next;
   }
   if (n_tags_altered > 0) {
-    way->flags |= OSM_FLAG_DIRTY;
+    OSM_FLAGS(way) |= OSM_FLAG_DIRTY;
   }
   return n_tags_altered;
 }
@@ -2125,7 +2205,7 @@ osm_way_reverse_direction_sensitive_roles(osm_t *osm, way_t *way) {
   guint n_roles_flipped = 0;
 
   for (; rel_chain != NULL; rel_chain = rel_chain->next) {
-    char *type = osm_tag_get_by_key(rel_chain->relation->tag, "type");
+    char *type = osm_tag_get_by_key(OSM_TAG(rel_chain->relation), "type");
     
     // Route relations; http://wiki.openstreetmap.org/wiki/Relation:route
     if (strcasecmp(type, "route") == 0) {
@@ -2138,7 +2218,7 @@ osm_way_reverse_direction_sensitive_roles(osm_t *osm, way_t *way) {
             break;
         }
         if (member->object.type == WAY_ID) {
-          if (member->object.id == way->id)
+          if (member->object.id == OSM_ID(way))
             break;
         }
       }
@@ -2151,13 +2231,13 @@ osm_way_reverse_direction_sensitive_roles(osm_t *osm, way_t *way) {
       else if (strcasecmp(member->role, DS_ROUTE_FORWARD) == 0) {
         g_free(member->role);
         member->role = g_strdup(DS_ROUTE_REVERSE);
-        rel_chain->relation->flags |= OSM_FLAG_DIRTY;
+        OSM_FLAGS(rel_chain->relation) |= OSM_FLAG_DIRTY;
         ++n_roles_flipped;
       }
       else if (strcasecmp(member->role, DS_ROUTE_REVERSE) == 0) {
         g_free(member->role);
         member->role = g_strdup(DS_ROUTE_FORWARD);
-        rel_chain->relation->flags |= OSM_FLAG_DIRTY;
+        OSM_FLAGS(rel_chain->relation) |= OSM_FLAG_DIRTY;
         ++n_roles_flipped;
       }
 
@@ -2214,22 +2294,20 @@ void osm_way_rotate(way_t *way, gint offset) {
   }
 }
 
-tag_t *osm_tags_copy(tag_t *src_tag, gboolean update_creator) {
+tag_t *osm_tags_copy(tag_t *src_tag) {
   tag_t *new_tags = NULL;
   tag_t **dst_tag = &new_tags;
 
   while(src_tag) {
-    *dst_tag = g_new0(tag_t, 1);
-    (*dst_tag)->key = g_strdup(src_tag->key);
-    if(update_creator && (strcasecmp(src_tag->key, "created_by") == 0))
-      (*dst_tag)->value = g_strdup(PACKAGE " v" VERSION);
-    else
+    if(!osm_is_creator_tag(src_tag)) {
+      *dst_tag = g_new0(tag_t, 1);
+      (*dst_tag)->key = g_strdup(src_tag->key);
       (*dst_tag)->value = g_strdup(src_tag->value);
-
-    dst_tag = &(*dst_tag)->next;
+      dst_tag = &(*dst_tag)->next;
+    }
     src_tag = src_tag->next;
   }
-
+  
   return new_tags;
 }
 
@@ -2254,15 +2332,8 @@ char *osm_object_type_string(object_t *object) {
   return NULL;
 }
 
-char *osm_object_get_name(object_t *object) {
-  tag_t *tags = osm_object_get_tags(object);
-
-  if(!tags) return NULL;
-  return osm_tag_get_by_key(tags, "name");
-}
-
 /* try to get an as "speaking" description of the object as possible */
-char *osm_object_get_speaking_name(object_t *object) {
+char *osm_object_get_name(object_t *object) {
   char *ret = NULL;
   tag_t *tags = osm_object_get_tags(object);
 
@@ -2354,14 +2425,12 @@ char *osm_object_string(object_t *object) {
     return g_strdup_printf("%s #<unspec>", type_str);
     break;
   case NODE:
-    return g_strdup_printf("%s #" ITEM_ID_FORMAT, type_str, object->node->id);
-    break;
   case WAY:
-    return g_strdup_printf("%s #" ITEM_ID_FORMAT, type_str, object->way->id);
-    break;
   case RELATION:
+    g_assert(object->ptr);
     return g_strdup_printf("%s #" ITEM_ID_FORMAT, type_str, 
-			   object->relation->id);
+			   OBJECT_ID(*object));
+    break;
     break;
   case NODE_ID:
   case WAY_ID:
@@ -2380,13 +2449,9 @@ char *osm_object_id_string(object_t *object) {
     return NULL;
     break;
   case NODE:
-    return g_strdup_printf("#"ITEM_ID_FORMAT, object->node->id);
-    break;
   case WAY:
-    return g_strdup_printf("#"ITEM_ID_FORMAT, object->way->id);
-    break;
   case RELATION:
-    return g_strdup_printf("#"ITEM_ID_FORMAT, object->relation->id);
+    return g_strdup_printf("#"ITEM_ID_FORMAT, OBJECT_ID(*object));
     break;
   case NODE_ID:
   case WAY_ID:
@@ -2397,29 +2462,26 @@ char *osm_object_id_string(object_t *object) {
   return NULL;
 }
 
+
+gboolean osm_object_is_real(object_t *object) {
+  return((object->type == NODE) ||
+	 (object->type == WAY)  ||
+	 (object->type == RELATION));
+}
+
 tag_t *osm_object_get_tags(object_t *object) {
   if(!object) return NULL;
+  if(!osm_object_is_real(object)) return NULL;
+  return OBJECT_TAG(*object);
+}
 
-  switch(object->type) {
-  case ILLEGAL:
-    return NULL;
-    break;
-  case NODE:
-    return object->node->tag;
-    break;
-  case WAY:
-    return object->way->tag;
-    break;
-  case RELATION:
-    return object->relation->tag;
-    break;
-  case NODE_ID:
-  case WAY_ID:
-  case RELATION_ID:
-    return NULL;
-    break;
-  }
-  return NULL;
+
+item_id_t osm_object_get_id(object_t *object) {
+  if(!object) return ID_ILLEGAL;
+
+  if(object->type == ILLEGAL)     return ID_ILLEGAL;
+  if(osm_object_is_real(object))  return OBJECT_ID(*object);
+  return object->id;
 }
 
 
@@ -2434,27 +2496,21 @@ gint osm_relation_members_num(relation_t *relation) {
 }
 
 void osm_object_set_flags(object_t *object, int set, int clr) {
-
-  switch(object->type) {
-  case NODE:
-    object->node->flags |=  set;
-    object->node->flags &= ~clr;
-    break;
-
-  case WAY:
-    object->way->flags |=  set;
-    object->way->flags &= ~clr;
-    break;
-
-  case RELATION:
-    object->relation->flags |=  set;
-    object->relation->flags &= ~clr;
-    break;
-
-  default:
-    g_assert(0);
-    break;
-  }
+  g_assert(osm_object_is_real(object));
+  OBJECT_FLAGS(*object) |=  set;
+  OBJECT_FLAGS(*object) &= ~clr;
 }
 
+gboolean osm_object_is_same(object_t *obj1, object_t *obj2) {
+  item_id_t id1 = osm_object_get_id(obj1);
+  item_id_t id2 = osm_object_get_id(obj2);
+
+  if(id1 == ID_ILLEGAL) return FALSE;
+  if(id2 == ID_ILLEGAL) return FALSE;
+  if(obj1->type != obj2->type) return FALSE;
+
+  return(id1 == id2);
+}
+
+
 // vim:et:ts=8:sw=2:sts=2:ai
